@@ -30,8 +30,6 @@ eval "$(pyenv virtualenv-init -)"
 # ensure python points to python3
 ln -s /usr/bin/python3 /usr/bin/python
 
-export CRYPTOGRAPHY_DONT_BUILD_RUST=1 # to avoid issue when building Cryptography python module
-
 # set the global Python version
 pyenv global $PYTHON_VERSION
 
@@ -129,7 +127,7 @@ CONFIG
     "MS_MAP_NO_PATH" "1"
   END
   MAPS
-    "MYMAPFILE" "wfs_simple.map"
+    "MYMAPFILE" "/tmp/wfs_simple.map"
   END
 END
 EOF
