@@ -73,6 +73,9 @@ static int msDrawRasterLayerGDAL_16BitClassification(
  * rasterBufferObj setting macros.
  */
 
+#define GAMMA_CORRECT(color, gamma)                                            \
+  (int)(0.5 + 255 * pow((color) / 255.0, (gamma)))
+
 /************************************************************************/
 /*                       msDrawRasterLayerGDAL()                        */
 /************************************************************************/
@@ -127,7 +130,7 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
   /* -------------------------------------------------------------------- */
   /*      Test the image format instead of the map format.                */
   /*      Normally the map format and the image format should be the      */
-  /*      same but In somes cases like swf and pdf support, a temporary   */
+  /*      same but In some cases like swf and pdf support, a temporary   */
   /*      GD image object is created and used to render raster layers     */
   /*      and then dumped into the SWF or the PDF file.                   */
   /* -------------------------------------------------------------------- */
@@ -541,6 +544,9 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
     return -1;
   }
 
+  const char *sgamma = msLayerGetProcessingKey(layer, "GAMMA");
+  const double gamma = sgamma ? CPLAtof(sgamma) : 1.0;
+
   /*
    * Setup the mapping between source eight bit pixel values, and the
    * output images color table.  There are two general cases, where the
@@ -654,6 +660,12 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
             rb_cmap[2][j] = pixel.blue;
             rb_cmap[3][j] = 255;
           }
+
+          if (gamma != 1.0) {
+            rb_cmap[0][j] = GAMMA_CORRECT(rb_cmap[0][j], gamma);
+            rb_cmap[1][j] = GAMMA_CORRECT(rb_cmap[1][j], gamma);
+            rb_cmap[2][j] = GAMMA_CORRECT(rb_cmap[2][j], gamma);
+          }
         }
       }
     }
@@ -684,6 +696,11 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
         rb_cmap[1][j] = sEntry.c2;
         rb_cmap[2][j] = sEntry.c3;
         rb_cmap[3][j] = sEntry.c4;
+        if (gamma != 1.0) {
+          rb_cmap[0][j] = GAMMA_CORRECT(rb_cmap[0][j], gamma);
+          rb_cmap[1][j] = GAMMA_CORRECT(rb_cmap[1][j], gamma);
+          rb_cmap[2][j] = GAMMA_CORRECT(rb_cmap[2][j], gamma);
+        }
       }
     }
   }
@@ -825,21 +842,33 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
       for (int j = dst_xoff; j < dst_xoff + dst_xsize; j++, k++) {
         if (SKIP_MASK(j, i) || (pbIsNoDataBuffer && pbIsNoDataBuffer[k]))
           continue;
-        if (MS_VALID_COLOR(layer->offsite) &&
-            pabyRaw1[k] == layer->offsite.red &&
-            pabyRaw2[k] == layer->offsite.green &&
-            pabyRaw3[k] == layer->offsite.blue)
+        int red = pabyRaw1[k];
+        int green = pabyRaw2[k];
+        int blue = pabyRaw3[k];
+        if (MS_VALID_COLOR(layer->offsite) && red == layer->offsite.red &&
+            green == layer->offsite.green && blue == layer->offsite.blue)
           continue;
 
-        if (bHaveRGBNoData && pabyRaw1[k] == nNoData1 &&
-            pabyRaw2[k] == nNoData2 && pabyRaw3[k] == nNoData3)
+        if (bHaveRGBNoData && red == nNoData1 && green == nNoData2 &&
+            blue == nNoData3)
           continue;
 
         if (pabyRawAlpha == NULL || pabyRawAlpha[k] == 255) {
-          RB_SET_PIXEL(rb, j, i, pabyRaw1[k], pabyRaw2[k], pabyRaw3[k], 255);
+          if (gamma != 1.0) {
+            red = GAMMA_CORRECT(red, gamma);
+            green = GAMMA_CORRECT(green, gamma);
+            blue = GAMMA_CORRECT(blue, gamma);
+          }
+
+          RB_SET_PIXEL(rb, j, i, red, green, blue, 255);
         } else if (pabyRawAlpha[k] != 0) {
-          RB_MIX_PIXEL(rb, j, i, pabyRaw1[k], pabyRaw2[k], pabyRaw3[k],
-                       pabyRawAlpha[k]);
+          if (gamma != 1.0) {
+            red = GAMMA_CORRECT(red, gamma);
+            green = GAMMA_CORRECT(green, gamma);
+            blue = GAMMA_CORRECT(blue, gamma);
+          }
+
+          RB_MIX_PIXEL(rb, j, i, red, green, blue, pabyRawAlpha[k]);
         }
       }
     }
@@ -1162,7 +1191,7 @@ static int ApplyLUT(int iColorIndex, const char *lut_def, const void *pInBuffer,
 
   if (lut_def == NULL) {
     if (pInBuffer != pabyOutBuffer) {
-      GDALCopyWords((void *)pInBuffer, eDT, GDALGetDataTypeSize(eDT) / 8,
+      GDALCopyWords(pInBuffer, eDT, GDALGetDataTypeSizeBytes(eDT),
                     pabyOutBuffer, GDT_Byte, 1, nPixelCount);
     }
     return 0;
@@ -1303,7 +1332,7 @@ static int LoadGDALImages(GDALDatasetH hDS, int band_numbers[4], int band_count,
       result_code =
           ApplyLUT(iColorIndex + 1, papszLUTs[iColorIndex],
                    (GByte *)pBuffer + dst_xsize * dst_ysize * iColorIndex *
-                                          (GDALGetDataTypeSize(eDT) / 8),
+                                          GDALGetDataTypeSizeBytes(eDT),
                    eDT, pabyWholeBuffer + dst_xsize * dst_ysize * iColorIndex,
                    dst_xsize * dst_ysize);
     }
@@ -1322,7 +1351,7 @@ static int LoadGDALImages(GDALDatasetH hDS, int band_numbers[4], int band_count,
   /*      We need to do some scaling.  Will load into either a 16bit      */
   /*      unsigned or a floating point buffer depending on the source     */
   /*      data.  We offer a special case for 16U data because it is       */
-  /*      common and it is a substantial win to avoid alot of floating    */
+  /*      common and it is a substantial win to avoid a lot of floating    */
   /*      point operations on it.                                         */
   /* -------------------------------------------------------------------- */
   /* TODO */
@@ -1370,12 +1399,14 @@ static int LoadGDALImages(GDALDatasetH hDS, int band_numbers[4], int band_count,
   /* -------------------------------------------------------------------- */
   for (iColorIndex = 0; iColorIndex < band_count; iColorIndex++) {
     unsigned char *pabyBuffer;
-    double dfScaleMin = 0.0, dfScaleMax = 255.0, dfScaleRatio, dfNoDataValue;
+    double dfScaleMin = 0.0, dfScaleMax = 255.0, dfScaleRatio = 0,
+           dfNoDataValue;
     const char *pszScaleInfo;
     float *pafRawData;
     int nPixelCount = dst_xsize * dst_ysize, i, bGotNoData = FALSE;
     GDALRasterBandH hBand = GDALGetRasterBand(hDS, band_numbers[iColorIndex]);
     pszScaleInfo = CSLFetchNameValue(layer->processing, "SCALE");
+    const bool bIsAllBandScale = pszScaleInfo != NULL;
     if (pszScaleInfo == NULL) {
       char szBandScalingName[20];
 
@@ -1388,7 +1419,17 @@ static int LoadGDALImages(GDALDatasetH hDS, int band_numbers[4], int band_count,
 
       papszTokens = CSLTokenizeStringComplex(pszScaleInfo, " ,", FALSE, FALSE);
       if (CSLCount(papszTokens) == 1 && EQUAL(papszTokens[0], "AUTO")) {
-        dfScaleMin = dfScaleMax = 0.0;
+        if (bIsAllBandScale &&
+            GDALGetRasterColorInterpretation(hBand) == GCI_AlphaBand &&
+            GDALGetRasterDataType(hBand) == GDT_Byte) {
+          // Do not try to scale the alpha band, as it is going to produce
+          // wrong results
+          dfScaleMin = 0.0;
+          dfScaleMax = 255.0;
+          dfScaleRatio = 1.0;
+        } else {
+          dfScaleMin = dfScaleMax = 0.0;
+        }
       } else if (CSLCount(papszTokens) != 2) {
         free(pafWholeRawData);
         CSLDestroy(papszTokens);
@@ -1451,7 +1492,8 @@ static int LoadGDALImages(GDALDatasetH hDS, int band_numbers[4], int band_count,
     /* -------------------------------------------------------------------- */
     /*      Now process the data.                                           */
     /* -------------------------------------------------------------------- */
-    dfScaleRatio = 256.0 / (dfScaleMax - dfScaleMin);
+    if (dfScaleRatio == 0.0)
+      dfScaleRatio = 256.0 / (dfScaleMax - dfScaleMin);
     pabyBuffer = pabyWholeBuffer + iColorIndex * nPixelCount;
 
     if (iColorIndex == 0 && bGotNoData)
@@ -1577,7 +1619,7 @@ int msGetGDALGeoTransform(GDALDatasetH hDS, mapObj *map, layerObj *layer,
         fileExtension = msStrdup(strrchr(szPath, '.') + 1);
       }
     }
-    /* common behaviour with worldfile generated from basename + .wld */
+    /* common behavior with worldfile generated from basename + .wld */
     else {
       fullPath = GDALGetDescription(hDS);
       fileExtension = msStrdup("wld");
@@ -1589,7 +1631,8 @@ int msGetGDALGeoTransform(GDALDatasetH hDS, mapObj *map, layerObj *layer,
     if (success && layer->debug >= MS_DEBUGLEVEL_VVV) {
       msDebug("Worldfile location: %s.\n", fullPath);
     } else if (layer->debug >= MS_DEBUGLEVEL_VVV) {
-      msDebug("Failed using worldfile location: %s.\n", fullPath);
+      msDebug("Failed using worldfile location: %s.\n",
+              fullPath ? fullPath : "(null)");
     }
 
     msFree(fileExtension);
@@ -1680,7 +1723,7 @@ msDrawRasterLayerGDAL_RawMode(mapObj *map, layerObj *layer, imageObj *image,
   int *band_list, band_count;
   int i, j, k, band;
   CPLErr eErr;
-  float *f_nodatas = NULL;
+  double *d_nodatas = NULL;
   unsigned char *b_nodatas = NULL;
   GInt16 *i_nodatas = NULL;
   int got_nodata = FALSE;
@@ -1710,6 +1753,8 @@ msDrawRasterLayerGDAL_RawMode(mapObj *map, layerObj *layer, imageObj *image,
     eDataType = GDT_Int16;
   else if (image->format->imagemode == MS_IMAGEMODE_FLOAT32)
     eDataType = GDT_Float32;
+  else if (image->format->imagemode == MS_IMAGEMODE_FLOAT64)
+    eDataType = GDT_Float64;
   else if (image->format->imagemode == MS_IMAGEMODE_BYTE)
     eDataType = GDT_Byte;
   else
@@ -1735,11 +1780,12 @@ msDrawRasterLayerGDAL_RawMode(mapObj *map, layerObj *layer, imageObj *image,
   /* -------------------------------------------------------------------- */
   /*      Do we have nodata values?                                       */
   /* -------------------------------------------------------------------- */
-  f_nodatas = (float *)calloc(sizeof(float), band_count);
-  if (f_nodatas == NULL) {
+
+  d_nodatas = (double *)calloc(band_count, sizeof(double));
+  if (d_nodatas == NULL) {
     msSetError(MS_MEMERR, "%s: %d: Out of memory allocating %u bytes.\n",
                "msDrawRasterLayerGDAL_RawMode()", __FILE__, __LINE__,
-               (unsigned int)(sizeof(float) * band_count));
+               (unsigned int)(sizeof(double) * band_count));
     free(band_list);
     return -1;
   }
@@ -1748,45 +1794,45 @@ msDrawRasterLayerGDAL_RawMode(mapObj *map, layerObj *layer, imageObj *image,
       (layer->offsite.red != -1 || layer->offsite.green != -1 ||
        layer->offsite.blue != -1)) {
     if (band_count > 0)
-      f_nodatas[0] = layer->offsite.red;
+      d_nodatas[0] = layer->offsite.red;
     if (band_count > 1)
-      f_nodatas[1] = layer->offsite.green;
+      d_nodatas[1] = layer->offsite.green;
     if (band_count > 2)
-      f_nodatas[2] = layer->offsite.blue;
+      d_nodatas[2] = layer->offsite.blue;
     got_nodata = TRUE;
   }
 
   if (!got_nodata) {
     got_nodata = TRUE;
     for (band = 0; band < band_count && got_nodata; band++) {
-      f_nodatas[band] = msGetGDALNoDataValue(
+      d_nodatas[band] = msGetGDALNoDataValue(
           layer, GDALGetRasterBand(hDS, band_list[band]), &got_nodata);
     }
   }
 
   if (!got_nodata) {
-    free(f_nodatas);
-    f_nodatas = NULL;
+    free(d_nodatas);
+    d_nodatas = NULL;
   } else if (eDataType == GDT_Byte) {
-    b_nodatas = (unsigned char *)f_nodatas;
+    b_nodatas = (unsigned char *)d_nodatas;
     for (band = 0; band < band_count; band++)
-      b_nodatas[band] = (unsigned char)f_nodatas[band];
+      b_nodatas[band] = (unsigned char)d_nodatas[band];
   } else if (eDataType == GDT_Int16) {
-    i_nodatas = (GInt16 *)f_nodatas;
+    i_nodatas = (GInt16 *)d_nodatas;
     for (band = 0; band < band_count; band++)
-      i_nodatas[band] = (GInt16)f_nodatas[band];
+      i_nodatas[band] = (GInt16)d_nodatas[band];
   }
 
   /* -------------------------------------------------------------------- */
   /*      Allocate buffer, and read data into it.                         */
   /* -------------------------------------------------------------------- */
   pBuffer = malloc(((size_t)dst_xsize) * dst_ysize * image->format->bands *
-                   (GDALGetDataTypeSize(eDataType) / 8));
+                   GDALGetDataTypeSizeBytes(eDataType));
   if (pBuffer == NULL) {
     msSetError(MS_MEMERR, "Allocating work image of size %dx%d failed.",
                "msDrawRasterLayerGDAL()", dst_xsize, dst_ysize);
     free(band_list);
-    free(f_nodatas);
+    free(d_nodatas);
     return -1;
   }
 
@@ -1800,7 +1846,7 @@ msDrawRasterLayerGDAL_RawMode(mapObj *map, layerObj *layer, imageObj *image,
     msSetError(MS_IOERR, "GDALRasterIO() failed: %s",
                "msDrawRasterLayerGDAL_RawMode()", CPLGetLastErrorMsg());
     free(pBuffer);
-    free(f_nodatas);
+    free(d_nodatas);
     return -1;
   }
 
@@ -1829,7 +1875,7 @@ msDrawRasterLayerGDAL_RawMode(mapObj *map, layerObj *layer, imageObj *image,
           int off = j + i * image->width + band * image->width * image->height;
           int off_mask = j + i * image->width;
 
-          if ((f_nodatas && ((float *)pBuffer)[k] == f_nodatas[band]) ||
+          if ((d_nodatas && ((float *)pBuffer)[k] == d_nodatas[band]) ||
               SKIP_MASK(j, i)) {
             k++;
             continue;
@@ -1838,6 +1884,20 @@ msDrawRasterLayerGDAL_RawMode(mapObj *map, layerObj *layer, imageObj *image,
           image->img.raw_float[off] = ((float *)pBuffer)[k++];
           MS_SET_BIT(image->img_mask, off_mask);
         }
+      } else if (image->format->imagemode == MS_IMAGEMODE_FLOAT64) {
+        for (j = dst_xoff; j < dst_xoff + dst_xsize; j++) {
+          int off = j + i * image->width + band * image->width * image->height;
+          int off_mask = j + i * image->width;
+
+          if ((d_nodatas && ((double *)pBuffer)[k] == d_nodatas[band]) ||
+              SKIP_MASK(j, i)) {
+            k++;
+            continue;
+          }
+
+          image->img.raw_double[off] = ((double *)pBuffer)[k++];
+          MS_SET_BIT(image->img_mask, off_mask);
+        }
       } else if (image->format->imagemode == MS_IMAGEMODE_BYTE) {
         for (j = dst_xoff; j < dst_xoff + dst_xsize; j++) {
           int off = j + i * image->width + band * image->width * image->height;
@@ -1857,7 +1917,7 @@ msDrawRasterLayerGDAL_RawMode(mapObj *map, layerObj *layer, imageObj *image,
   }
 
   free(pBuffer);
-  free(f_nodatas);
+  free(d_nodatas);
 
   return 0;
 }
@@ -2065,6 +2125,9 @@ static int msDrawRasterLayerGDAL_16BitClassification(
     msGettimeofday(&starttime, NULL);
   }
 
+  const char *sgamma = msLayerGetProcessingKey(layer, "GAMMA");
+  const double gamma = sgamma ? CPLAtof(sgamma) : 1.0;
+
   lastC = -1;
   for (i = 0; i < nBucketCount; i++) {
     double dfOriginalValue;
@@ -2118,6 +2181,12 @@ static int msDrawRasterLayerGDAL_16BitClassification(
           rb_cmap[2][i] = layer->class[c]->styles[styleindex]->color.blue;
           rb_cmap[3][i] =
               (255 * layer->class[c] -> styles[styleindex] -> opacity / 100);
+
+          if (gamma != 1.0) {
+            rb_cmap[0][i] = GAMMA_CORRECT(rb_cmap[0][i], gamma);
+            rb_cmap[1][i] = GAMMA_CORRECT(rb_cmap[1][i], gamma);
+            rb_cmap[2][i] = GAMMA_CORRECT(rb_cmap[2][i], gamma);
+          }
         }
       }
     }
