@@ -127,6 +127,7 @@ void msHTTPInitRequestObj(httpRequestObj *pasReqInfo, int numRequests) {
     pasReqInfo[i].pszPostRequest = NULL;
     pasReqInfo[i].pszPostContentType = NULL;
     pasReqInfo[i].pszOutputFile = NULL;
+    pasReqInfo[i].pszEPSG = NULL;
     pasReqInfo[i].nLayerId = 0;
     pasReqInfo[i].nTimeout = 0;
     pasReqInfo[i].nMaxBytes = 0;
@@ -174,6 +175,10 @@ void msHTTPFreeRequestObj(httpRequestObj *pasReqInfo, int numRequests) {
       free(pasReqInfo[i].pszOutputFile);
     pasReqInfo[i].pszOutputFile = NULL;
 
+    if (pasReqInfo[i].pszEPSG)
+      free(pasReqInfo[i].pszEPSG);
+    pasReqInfo[i].pszEPSG = NULL;
+
     if (pasReqInfo[i].pszContentType)
       free(pasReqInfo[i].pszContentType);
     pasReqInfo[i].pszContentType = NULL;
@@ -290,7 +295,7 @@ long msGetCURLAuthType(enum MS_HTTP_AUTH_TYPE authType) {
  * msPrepareWMSLayerRequest() to handle proxy / http auth for requests
  *
  * Return value:
- * MS_SUCCESS if all requests completed succesfully.
+ * MS_SUCCESS if all requests completed successfully.
  * MS_FAILURE if a fatal error happened
  **********************************************************************/
 int msHTTPAuthProxySetup(hashTableObj *mapmd, hashTableObj *lyrmd,
@@ -845,7 +850,7 @@ int msHTTPExecuteRequests(httpRequestObj *pasReqInfo, int numRequests,
 
     if (!MS_HTTP_SUCCESS(psReq->nStatus)) {
       /* Set status to MS_DONE to indicate that transfers were  */
-      /* completed but may not be succesfull */
+      /* completed but may not be successful */
       nStatus = MS_DONE;
 
       if (psReq->nStatus == -(CURLE_OPERATION_TIMEOUTED)) {
