@@ -31,6 +31,7 @@
 #define MAPOWS_H
 
 #include "maphttp.h"
+#include <stdbool.h>
 #include <time.h>
 
 /* This is the URL to the official OGC Schema Repository. We use it by
@@ -179,10 +180,6 @@ MS_DLL_EXPORT int msOWSParseRequestMetadata(const char *metadata,
 #define OWS_0_1_6 0x000106
 #define OWS_0_1_7 0x000107
 #define OWS_1_0_0 0x010000
-#define OWS_1_0_1 0x010001
-#define OWS_1_0_6 0x010006
-#define OWS_1_0_7 0x010007
-#define OWS_1_0_8 0x010008
 #define OWS_1_1_0 0x010100
 #define OWS_1_1_1 0x010101
 #define OWS_1_1_2 0x010102
@@ -355,6 +352,7 @@ outputFormatObj *msOwsIsOutputFormatValid(mapObj *map, const char *format,
                                           hashTableObj *metadata,
                                           const char *namespaces,
                                           const char *name);
+bool msOWSStrictCompliance(mapObj *map);
 #endif /* #if any wxs service enabled */
 
 /*====================================================================
