@@ -574,6 +574,7 @@ int msValidateContexts(mapObj *map) {
 int msEvalContext(mapObj *map, layerObj *layer, char *context) {
   int i, status;
   char *tag = NULL;
+  tokenListNodeObjPtr token = NULL;
 
   expressionObj e;
   parseObj p;
@@ -609,6 +610,24 @@ int msEvalContext(mapObj *map, layerObj *layer, char *context) {
 
   msTokenizeExpression(&e, NULL, NULL);
 
+  /*
+   * We'll check for binding tokens in the token list. Since there is no shape
+   * to bind to, the parser will crash if any are present.
+   * This is one way to catch references to non-existent layers.
+   */
+  for (token = e.tokens; token; token = token->next) {
+    if (token->token == MS_TOKEN_BINDING_DOUBLE ||
+        token->token == MS_TOKEN_BINDING_STRING ||
+        token->token == MS_TOKEN_BINDING_TIME) {
+      msSetError(MS_PARSEERR,
+                 "A non-existent layer is referenced in a LAYER REQUIRES or "
+                 "LABELREQUIRES expression: %s",
+                 "msEvalContext()", e.string);
+      msFreeExpression(&e);
+      return MS_FALSE;
+    }
+  }
+
   p.shape = NULL;
   p.expr = &e;
   p.expr->curtoken = p.expr->tokens; /* reset */
@@ -804,13 +823,10 @@ int msShapeGetNextClass(int currentclass, layerObj *layer, mapObj *map,
       if (iclass < 0 || iclass >= layer->numclasses)
         continue; /* this should never happen but just in case */
 
-      if (map->scaledenom > 0) { /* verify scaledenom here  */
-        if ((layer->class[iclass] -> maxscaledenom > 0) &&
-            (map->scaledenom > layer->class[iclass] -> maxscaledenom))
-          continue; /* can skip this one, next class */
-        if ((layer->class[iclass] -> minscaledenom > 0) &&
-            (map->scaledenom <= layer->class[iclass] -> minscaledenom))
-          continue; /* can skip this one, next class */
+      if (!msScaleInBounds(map->scaledenom,
+                           layer->class[iclass] -> minscaledenom,
+                           layer -> class[iclass] -> maxscaledenom)) {
+        continue;
       }
 
       /* verify the minfeaturesize */
@@ -1751,6 +1767,7 @@ imageObj *msImageCreate(int width, int height, outputFormatObj *format,
   } else if (MS_RENDERER_RAWDATA(format)) {
     if (format->imagemode != MS_IMAGEMODE_INT16 &&
         format->imagemode != MS_IMAGEMODE_FLOAT32 &&
+        format->imagemode != MS_IMAGEMODE_FLOAT64 &&
         format->imagemode != MS_IMAGEMODE_BYTE) {
       msSetError(MS_IMGERR,
                  "Attempt to use illegal imagemode with rawdata renderer.",
@@ -1771,6 +1788,9 @@ imageObj *msImageCreate(int width, int height, outputFormatObj *format,
     else if (format->imagemode == MS_IMAGEMODE_FLOAT32)
       image->img.raw_float = (float *)msSmallCalloc(
           sizeof(float), ((size_t)width) * height * format->bands);
+    else if (format->imagemode == MS_IMAGEMODE_FLOAT64)
+      image->img.raw_double = (double *)msSmallCalloc(
+          sizeof(double), ((size_t)width) * height * format->bands);
     else if (format->imagemode == MS_IMAGEMODE_BYTE)
       image->img.raw_byte = (unsigned char *)msSmallCalloc(
           sizeof(unsigned char), ((size_t)width) * height * format->bands);
@@ -1815,6 +1835,10 @@ imageObj *msImageCreate(int width, int height, outputFormatObj *format,
         float nv = atof(nullvalue);
         for (; i > 0;)
           image->img.raw_float[--i] = nv;
+      } else if (format->imagemode == MS_IMAGEMODE_FLOAT64) {
+        double nv = atof(nullvalue);
+        for (; i > 0;)
+          image->img.raw_double[--i] = nv;
       } else if (format->imagemode == MS_IMAGEMODE_BYTE) {
         unsigned char nv = (unsigned char)atoi(nullvalue);
 
@@ -2569,7 +2593,7 @@ void *msSmallCalloc(size_t nCount, size_t nSize) {
 ** msBuildOnlineResource()
 **
 ** Try to build the online resource (mapserv URL) for this service.
-** "http://$(SERVER_NAME):$(SERVER_PORT)$(SCRIPT_NAME)?"
+** "http://$(SERVER_NAME):$(SERVER_PORT)$(SCRIPT_NAME)$(PATH_INFO)?"
 ** (+append the map=... param if it was explicitly passed in QUERY_STRING)
 **
 ** Returns a newly allocated string that should be freed by the caller or
@@ -2578,12 +2602,11 @@ void *msSmallCalloc(size_t nCount, size_t nSize) {
 char *msBuildOnlineResource(mapObj *map, cgiRequestObj *req) {
   (void)map;
   char *online_resource = NULL;
-  const char *value, *hostname, *port, *script, *protocol = "http",
-                                                *mapparam = NULL;
+  const char *value, *protocol = "http", *mapparam = NULL;
   char **hostname_array = NULL;
   int mapparam_len = 0, hostname_array_len = 0;
 
-  hostname = getenv("HTTP_X_FORWARDED_HOST");
+  const char *hostname = getenv("HTTP_X_FORWARDED_HOST");
   if (!hostname)
     hostname = getenv("SERVER_NAME");
   else {
@@ -2593,11 +2616,16 @@ char *msBuildOnlineResource(mapObj *map, cgiRequestObj *req) {
     }
   }
 
-  port = getenv("HTTP_X_FORWARDED_PORT");
+  const char *port = getenv("HTTP_X_FORWARDED_PORT");
   if (!port)
     port = getenv("SERVER_PORT");
 
-  script = getenv("SCRIPT_NAME");
+  const char *pathinfo = getenv("PATH_INFO");
+  if (!pathinfo) {
+    pathinfo = "";
+  }
+
+  const char *script = getenv("SCRIPT_NAME");
 
   /* HTTPS is set by Apache to "on" in an HTTPS server ... if not set */
   /* then check SERVER_PORT: 443 is the default https port. */
@@ -2624,18 +2652,19 @@ char *msBuildOnlineResource(mapObj *map, cgiRequestObj *req) {
 
   if (hostname && port && script) {
     size_t buffer_size;
-    buffer_size =
-        strlen(hostname) + strlen(port) + strlen(script) + mapparam_len +
-        11; /* 11 comes from https://[host]:[port][scriptname]?[map]\0, i.e.
-               "https://:?\0" */
+    buffer_size = strlen(hostname) + strlen(port) + strlen(pathinfo) +
+                  strlen(script) + mapparam_len +
+                  11; /* 11 comes from
+                         https://[host]:[port][scriptname][pathinfo]?[map]\0,
+                         i.e. "https://:?\0" */
     online_resource = (char *)msSmallMalloc(buffer_size);
     if ((atoi(port) == 80 && strcmp(protocol, "http") == 0) ||
         (atoi(port) == 443 && strcmp(protocol, "https") == 0))
-      snprintf(online_resource, buffer_size, "%s://%s%s?", protocol, hostname,
-               script);
+      snprintf(online_resource, buffer_size, "%s://%s%s%s?", protocol, hostname,
+               script, pathinfo);
     else
-      snprintf(online_resource, buffer_size, "%s://%s:%s%s?", protocol,
-               hostname, port, script);
+      snprintf(online_resource, buffer_size, "%s://%s:%s%s%s?", protocol,
+               hostname, port, script, pathinfo);
 
     if (mapparam) {
       int baselen;
