@@ -243,7 +243,7 @@ static char *msWMSLayerGetFormat(layerObj *lp) {
   } else {
     msSetError(MS_WMSCONNERR,
                "Could not find a format that matches supported input "
-               "formats in wms_formatlist metdata in layer %s.  "
+               "formats in wms_formatlist metadata in layer %s.  "
                "Please either provide a valid CONNECTION URL, or "
                "provide the required layer metadata.\n",
                "msBuildWMSLayerURLBase()", lp->name);
@@ -262,7 +262,7 @@ static char *msWMSLayerGetFormat(layerObj *lp) {
  *   FORMAT
  *   TRANSPARENT
  *   STYLES
- *   QUERY_LAYERS (for queriable layers only)
+ *   QUERY_LAYERS (for queryable layers only)
  *
  * Returns a reference to a newly allocated string that should be freed
  * by the caller.
@@ -306,16 +306,9 @@ static int msBuildWMSLayerURLBase(mapObj *map, layerObj *lp,
 
   psWMSParams->onlineresource = msStrdup(pszOnlineResource);
 
-  if (strncmp(pszVersion, "1.0.7", 5) < 0)
-    pszVersionKeyword = "WMTVER";
-  else
-    pszVersionKeyword = "VERSION";
+  pszVersionKeyword = "VERSION";
 
   nVersion = msOWSParseVersionString(pszVersion);
-  /* WMS 1.0.8 is really just 1.1.0 */
-  if (nVersion == OWS_1_0_8)
-    nVersion = OWS_1_1_0;
-
   msSetWMSParamString(psWMSParams, pszVersionKeyword, pszVersion, MS_FALSE,
                       nVersion);
   msSetWMSParamString(psWMSParams, "SERVICE", "WMS", MS_FALSE, nVersion);
@@ -377,7 +370,7 @@ static int msBuildWMSLayerURLBase(mapObj *map, layerObj *lp,
 
   /* if  the metadata wms_sld_body is set to AUTO, we generate
    * the sld based on classes found in the map file and send
-   * it in the URL. If diffrent from AUTO, we are assuming that
+   * it in the URL. If different from AUTO, we are assuming that
    * it is a valid sld.
    */
   if (pszSLDBody) {
@@ -438,6 +431,7 @@ static int msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
       *pszSrsParamName = "SRS", *pszLayer = NULL, *pszQueryLayers = NULL,
       *pszUseStrictAxisOrder;
   rectObj bbox;
+  projectionObj layerRequestProjection;
   int bbox_width = map->width, bbox_height = map->height;
   int nVersion = OWS_VERSION_NOTSET;
   int bUseStrictAxisOrder = MS_FALSE; /* this is the assumption up to 1.1.0 */
@@ -486,12 +480,7 @@ static int msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
   }
 
   switch (nVersion) {
-  case OWS_1_0_8:
-    nVersion = OWS_1_1_0; /* 1.0.8 == 1.1.0 */
-    break;
   case OWS_1_0_0:
-  case OWS_1_0_1:
-  case OWS_1_0_7:
   case OWS_1_1_0:
   case OWS_1_1_1:
     /* All is good, this is a supported version. */
@@ -642,51 +631,11 @@ static int msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
     bFlipAxisOrder = MS_TRUE;
   }
 
-  /* ------------------------------------------------------------------
-   * Set layer SRS.
-   * ------------------------------------------------------------------ */
-  /* No need to set lp->proj if it's already set to the right EPSG code */
-  {
-    char *pszEPSGCodeFromLayer = NULL;
-    msOWSGetEPSGProj(&(lp->projection), NULL, "MO", MS_TRUE,
-                     &pszEPSGCodeFromLayer);
-    if (pszEPSGCodeFromLayer == NULL ||
-        strcasecmp(pszEPSG, pszEPSGCodeFromLayer) != 0) {
-      char *ows_srs = NULL;
-      msOWSGetEPSGProj(NULL, &(lp->metadata), "MO", MS_FALSE, &ows_srs);
-      /* no need to set lp->proj if it is already set and there is only
-      one item in the _srs metadata for this layer - we will assume
-      the projection block matches the _srs metadata (the search for ' '
-      in ows_srs is a test to see if there are multiple EPSG: codes) */
-      if (lp->projection.numargs == 0 || ows_srs == NULL ||
-          (strchr(ows_srs, ' ') != NULL)) {
-        // Reproject layer extent if lp-projection is set since we might change
-        // layer projection
-        if (msProjectionsDiffer(&(map->projection), &(lp->projection))) {
-          msProjectRect(&(lp->projection), &(map->projection), &(lp->extent));
-        }
-
-        if (strncasecmp(pszEPSG, "EPSG:", 5) == 0) {
-          char szProj[20];
-          snprintf(szProj, sizeof(szProj), "init=epsg:%s", pszEPSG + 5);
-          if (msLoadProjectionString(&(lp->projection), szProj) != 0) {
-            msFree(pszEPSGCodeFromLayer);
-            msFree(ows_srs);
-            free(pszEPSG);
-            return MS_FAILURE;
-          }
-        } else {
-          if (msLoadProjectionString(&(lp->projection), pszEPSG) != 0) {
-            msFree(pszEPSGCodeFromLayer);
-            msFree(ows_srs);
-            free(pszEPSG);
-            return MS_FAILURE;
-          }
-        }
-      }
-      msFree(ows_srs);
-    }
-    msFree(pszEPSGCodeFromLayer);
+  msInitProjection(&layerRequestProjection);
+  if (msLoadProjectionString(&layerRequestProjection, pszEPSG) != 0) {
+    free(pszEPSG);
+    msFreeProjection(&layerRequestProjection);
+    return MS_FAILURE;
   }
 
   /* ------------------------------------------------------------------
@@ -703,8 +652,8 @@ static int msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
   /* -------------------------------------------------------------------- */
   /*      Reproject if needed.                                            */
   /* -------------------------------------------------------------------- */
-  if (msProjectionsDiffer(&(map->projection), &(lp->projection))) {
-    msProjectRect(&(map->projection), &(lp->projection), &bbox);
+  if (msProjectionsDiffer(&(map->projection), &layerRequestProjection)) {
+    msProjectRect(&(map->projection), &layerRequestProjection, &bbox);
 
     /* -------------------------------------------------------------------- */
     /*      Sometimes our remote WMS only accepts square pixel              */
@@ -761,7 +710,8 @@ static int msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
                      &ows_srs);
 
     if (ows_srs && strchr(ows_srs, ' ') == NULL &&
-        msOWSGetLayerExtent(map, lp, "MO", &layer_rect) == MS_SUCCESS) {
+        msOWSGetLayerExtent(map, lp, "MO", &layer_rect) == MS_SUCCESS &&
+        !msProjectionsDiffer(&(lp->projection), &layerRequestProjection)) {
       /* fulloverlap */
       if (msRectContained(&bbox, &layer_rect)) {
         /* no changes */
@@ -844,7 +794,7 @@ static int msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
   if (nRequestType == WMS_GETFEATUREINFO) {
     char szBuf[100] = "";
 
-    if (nVersion >= OWS_1_0_7)
+    if (nVersion >= OWS_1_1_0)
       pszRequestParam = "GetFeatureInfo";
     else
       pszRequestParam = "feature_info";
@@ -854,8 +804,6 @@ static int msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
       pszExceptionsParam = "XML";
     else if (nVersion >= OWS_1_1_0) /* 1.1.0 to 1.1.0 */
       pszExceptionsParam = "application/vnd.ogc.se_xml";
-    else if (nVersion > OWS_1_0_0) /* 1.0.1 to 1.0.7 */
-      pszExceptionsParam = "SE_XML";
     else
       pszExceptionsParam = "WMS_XML";
 
@@ -940,7 +888,7 @@ static int msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
   } else { /* if (nRequestType == WMS_GETMAP) */
     char szBuf[100] = "";
 
-    if (nVersion >= OWS_1_0_7)
+    if (nVersion >= OWS_1_1_0)
       pszRequestParam = "GetMap";
     else
       pszRequestParam = "map";
@@ -983,7 +931,7 @@ static int msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
   }
 
   free(pszEPSG);
-
+  msFreeProjection(&layerRequestProjection);
   return MS_SUCCESS;
 
 #else
@@ -1041,6 +989,8 @@ int msPrepareWMSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
   char *pszURL = NULL, *pszHTTPCookieData = NULL;
   const char *pszTmp;
   rectObj bbox = {0};
+  projectionObj lyrRequestProjection;
+  const char *pszEPSG;
   int bbox_width = 0, bbox_height = 0;
   int nTimeout, bOkToMerge, bForceSeparateRequest, bCacheToDisk;
   wmsParamsObj sThisWMSParams;
@@ -1052,8 +1002,7 @@ int msPrepareWMSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
   msInitWmsParamsObj(&sThisWMSParams);
 
   /* ------------------------------------------------------------------
-   * Build the request URL, this will also set layer projection and
-   * compute BBOX in that projection.
+   * Build the request URL
    * ------------------------------------------------------------------ */
 
   switch (nRequestType) {
@@ -1084,12 +1033,31 @@ int msPrepareWMSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
     return MS_FAILURE;
   }
 
+  /* Load projection that we will request, it may differ from current layer
+   * projection */
+  char *srsKeys[] = {"SRS", "CRS"};
+  int i;
+  for (i = 0; i < 2; i++) {
+    pszEPSG = msLookupHashTable(sThisWMSParams.params, srsKeys[i]);
+    if (pszEPSG != NULL && strlen(pszEPSG) > 0)
+      break;
+  }
+
+  msInitProjection(&lyrRequestProjection);
+  if (pszEPSG == NULL || msLoadProjectionStringEPSG(&lyrRequestProjection,
+                                                    pszEPSG) != MS_SUCCESS) {
+    msFreeWmsParamsObj(&sThisWMSParams);
+    msFreeProjection(&lyrRequestProjection);
+    return MS_FAILURE;
+  }
+
   /* ------------------------------------------------------------------
    * Check if the request is empty, perhaps due to reprojection problems
    * or wms_extents restrictions.
    * ------------------------------------------------------------------ */
   if ((nRequestType == WMS_GETMAP) && (bbox_width == 0 || bbox_height == 0)) {
     msFreeWmsParamsObj(&sThisWMSParams);
+    msFreeProjection(&lyrRequestProjection);
     return MS_SUCCESS; /* No overlap. */
   }
 
@@ -1122,10 +1090,10 @@ int msPrepareWMSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
     /* Reproject latlonboundingbox to the selected SRS for the layer and */
     /* check if it overlaps the bbox that we calculated for the request */
 
-    msProjectRect(&(map->latlon), &(lp->projection), &ext);
+    msProjectRect(&(map->latlon), &lyrRequestProjection, &ext);
     if (!msRectOverlap(&bbox, &ext)) {
       /* No overlap... nothing to do */
-
+      msFreeProjection(&lyrRequestProjection);
       msFreeWmsParamsObj(&sThisWMSParams);
       return MS_SUCCESS; /* No overlap. */
     }
@@ -1161,6 +1129,8 @@ int msPrepareWMSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
       msSetError(MS_WMSERR,
                  "WEB.IMAGEPATH must be set to use WMS client connections.",
                  "msPrepareWMSLayerRequest()");
+      msFreeProjection(&lyrRequestProjection);
+      msFreeWmsParamsObj(&sThisWMSParams);
       return MS_FAILURE;
     }
   }
@@ -1303,6 +1273,7 @@ int msPrepareWMSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
     pasReqInfo[(*numRequests)].width = bbox_width;
     pasReqInfo[(*numRequests)].height = bbox_height;
     pasReqInfo[(*numRequests)].debug = lp->debug;
+    pasReqInfo[(*numRequests)].pszEPSG = msStrdup(pszEPSG);
 
     if (msHTTPAuthProxySetup(&(map->web.metadata), &(lp->metadata), pasReqInfo,
                              *numRequests, map, "MO") != MS_SUCCESS)
@@ -1326,6 +1297,8 @@ int msPrepareWMSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
     msFreeWmsParamsObj(&sThisWMSParams);
   }
 
+  msFreeProjection(&lyrRequestProjection);
+
   return MS_SUCCESS;
 
 #else
@@ -1352,6 +1325,8 @@ int msDrawWMSLayerLow(int nLayerId, httpRequestObj *pasReqInfo, int numRequests,
   char szPath[MS_MAXPATHLEN];
   int currenttype;
   int currentconnectiontype;
+  projectionObj currentprojectionobject;
+  rectObj currentextent;
   int numclasses;
   char *mem_filename = NULL;
   const char *pszTmp;
@@ -1475,12 +1450,35 @@ int msDrawWMSLayerLow(int nLayerId, httpRequestObj *pasReqInfo, int numRequests,
    * Prepare layer for drawing, reprojecting the image received from the
    * server if needed...
    * ------------------------------------------------------------------ */
+
+  // Aassign requested projection temporarily to layer
+  currentprojectionobject = lp->projection;
+  msInitProjection(&(lp->projection));
+  if (pasReqInfo[iReq].pszEPSG == NULL ||
+      msLoadProjectionString(&(lp->projection), pasReqInfo[iReq].pszEPSG) !=
+          MS_SUCCESS) {
+    msSetError(
+        MS_WMSERR, "WMS GetMap failed to load projection for layer '%s': %s.",
+        "msDrawWMSLayerLow()", (lp->name ? lp->name : "(null)"),
+        (pasReqInfo[iReq].pszEPSG ? pasReqInfo[iReq].pszEPSG : "(null)"));
+
+    // Restore layer projection
+    msFreeProjection(&(lp->projection));
+    lp->projection = currentprojectionobject;
+    if (!bIsEssential)
+      return MS_SUCCESS;
+    else
+      return MS_FAILURE;
+  }
+
   /* keep the current type that will be restored at the end of this  */
   /* function. */
   currenttype = lp->type;
   currentconnectiontype = lp->connectiontype;
+  currentextent = lp->extent;
   lp->type = MS_LAYER_RASTER;
   lp->connectiontype = MS_RASTER;
+  lp->extent = pasReqInfo[iReq].bbox;
 
   /* set the classes to 0 so that It won't do client side */
   /* classification if an sld was set. */
@@ -1494,12 +1492,9 @@ int msDrawWMSLayerLow(int nLayerId, httpRequestObj *pasReqInfo, int numRequests,
       msOWSLookupMetadata(&(lp->metadata), "MO", "sld_url"))
     lp->numclasses = 0;
 
-  if (lp->data)
-    free(lp->data);
-  if (mem_filename != NULL)
-    lp->data = mem_filename;
-  else
-    lp->data = msStrdup(pasReqInfo[iReq].pszOutputFile);
+  free(lp->data);
+  lp->data = mem_filename != NULL ? mem_filename
+                                  : msStrdup(pasReqInfo[iReq].pszOutputFile);
 
   const char *pszAllowedDrivers =
       msOWSLookupMetadata(&(lp->metadata), "MO", "allowed_gdal_drivers");
@@ -1575,6 +1570,10 @@ int msDrawWMSLayerLow(int nLayerId, httpRequestObj *pasReqInfo, int numRequests,
   /* restore prveious type */
   lp->type = currenttype;
   lp->connectiontype = currentconnectiontype;
+  lp->extent = currentextent;
+
+  msFreeProjection(&(lp->projection));
+  lp->projection = currentprojectionobject;
 
   /* restore previous numclasses */
   lp->numclasses = numclasses;
